@@ -69,7 +69,8 @@ arrayRemove = function(array, from, to) {
   return array.push.apply(array, rest);
 };
 
-mozmill = undefined; elementslib = undefined;
+mozmill = undefined; elementslib = undefined; modules = undefined;
+
 var loadTestResources = function () {
   if (mozmill == undefined) {
     mozmill = {};
@@ -108,7 +109,12 @@ var loadFile = function(path, collector) {
                   Ci: Components.interfaces,
                   Cu: Components.utils }
     });
-    return loader.require(mod);
+    if (modules != undefined) {
+        loader.modules = modules;
+    }
+    var retval = loader.require(mod);
+    modules = loader.modules;
+    return retval;
   }
   
   if (collector != undefined) {
@@ -117,18 +123,21 @@ var loadFile = function(path, collector) {
   }
   try {
     loader.loadSubScript(uri, module, "UTF-8");
-  } catch(e) {
-    events.fail(e);
+  } catch (e) {
+    events.fail({'exception' : e});
+
     var obj = {
-      'filename':path,
-      'passed':false,
-      'failed':true,
-      'passes':0,
-      'fails' :1,
-      'name'  :'Unknown Test',
+      'filename': path,
+      'passed': 0,
+      'failed': 1,
+      'passes': [ ],
+      'fails' : [{'exception' : {
+                     message: e.message,
+                     filename: e.filename,
+                     lineNumber: e.lineNumber}}],
+      'name'  : '<TOP_LEVEL>'
     };
     events.fireEvent('endTest', obj);
-    Components.utils.reportError(e);
   }
   
   module.__file__ = path;
@@ -231,6 +240,18 @@ events.pass = function (obj) {
   events.fireEvent('pass', obj);
 }
 events.fail = function (obj) {
+  var error = obj.exception;
+  if(error) {
+    // Error objects aren't enumerable https://bugzilla.mozilla.org/show_bug.cgi?id=637207
+    obj.exception = {
+      name: error.name,
+      message: error.message,
+      lineNumber: error.lineNumber,
+      fileName: error.fileName,
+      stack: error.stack
+    };
+  }
+  // a low level event, such as a keystroke, fails
   if (events.currentTest) {
     events.currentTest.__fails__.push(obj);
   }
